//
//  Copyright (C) 2011-2012 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using Footnote.Widgets;

class LLabel : Gtk.Label {
    public LLabel (string label) {
        this.set_halign (Gtk.Align.START);
        this.label = label;
    }
    public LLabel.indent (string label) {
        this (label);
        this.margin_left = 10;
    }
    public LLabel.markup (string label){
        this (label);
        this.use_markup = true;
    }
    public LLabel.r (string label){
        this.set_halign (Gtk.Align.END);
        this.label = label;
    }
    public LLabel.right_with_markup (string label){
        this.set_halign (Gtk.Align.END);
        this.use_markup = true;
        this.label = label;
    }
}


namespace Footnote {
    
    
    /*
     * Store for default settings in d-conf
     */
    public class FootnoteSettings : Granite.Services.Settings {
        
        public int window_width             {get; set;}
        public int window_height            {get; set;}
        public int sidebar_width            {get; set;}
        public int view_mode                {get; set;}
        public int miller_handle_position   {get; set;}
        
        public FootnoteSettings (){
            base ("org.pantheon.footnote");
        }
    }
    
    
    
    
    /**
     * the app and everything else
     **/
    public class FootnoteApp : Granite.Application {
    
        public struct Selected {
            public Notebook notebook;
            public Note     note;
        }
        
        public SQLHeavy.Database         db;
        public List<Notebook>            notebooks;
        public List<Note>                deleted_notes;
        
        public Gtk.Window                mainwindow;
        public Gtk.Box                   note_view;
        public EditView                  edit_view;
        public ElementaryWidgets.SideBar sidebar;
        public Gtk.Box                   right_box;
        public Granite.Widgets.Welcome   welcome;
        public Granite.Widgets.ModeButton view_mode;
        public Granite.Widgets.SidebarPaned hmain;
        
        public Granite.Widgets.SearchBar search;
        public Gtk.ToolButton            add_button;
        public Gtk.ToolButton            remove_button;
        public Gtk.ToolButton            edit_button;
        
        public Gtk.TreeIter              notebooks_iter;
        public Gtk.TreeIter              bookmarks_iter;
        
        public Gtk.ComboBoxText          years;
        
        public Gtk.ActionGroup           main_actions;
        private Gtk.UIManager            ui;
        public Selected                  selected;
        public bool                      showing_search;
        public bool                      fullscreened;
        public FootnoteSettings          settings;
        public Gtk.InfoBar               undo_bar;
        
        public int                       year;
        public int                       month;
        public GLib.List<int>            displayed_years; //list of currently displayed years
        
        construct {
            program_name = "Footnote";
            exec_name = "footnote";
            
            build_data_dir = Constants.DATADIR;
            build_pkg_data_dir = Constants.PKGDATADIR;
            build_release_name = Constants.RELEASE_NAME;
            build_version = Constants.VERSION;
            build_version_info = Constants.VERSION_INFO;
            
            app_years = "2011-2012";
            app_icon = "footnote";
            app_launcher = "footnote.desktop";
            application_id = "net.launchpad.Footnote";
            
            main_url = "https://code.launchpad.net/footnote";
            bug_url = "https://bugs.launchpad.net/footnote";
            help_url = "https://code.launchpad.net/footnote";
            translate_url = "https://translations.launchpad.net/footnote";
            
            about_authors = {"Tom Beckmann <tombeckmann@online.de>"};
            about_documenters = {"Tom Beckmann <tombeckmann@online.de>"};
            about_artists = {"Harvey Cabaguio"};
            about_comments = "Development release, not all features implemented";
            about_translators = "";
            about_license_type = Gtk.License.GPL_3_0;
        }
        
        const string ui_string = """
            <ui>
            <popup name="MenuItemTool">
                <menuitem name="Quit" action="Quit"/>
                <menuitem name="New Note" action="NewNote"/>
                <menuitem name="Delete Note" action="DeleteNote"/>
                <menuitem name="Set Bold" action="SetBold"/>
                <menuitem name="Strike Through" action="StrikeThrough"/>
                <menuitem name="Set Italic" action="SetItalic"/>
                <menuitem name="Fullscreen" action="Fullscreen"/>
            </popup>
            </ui>
        """;
        
        public FootnoteApp (){
            month = -1;
            year  = -1;
            
            notebooks = new List<Notebook> ();
            deleted_notes = new List<Note> ();
            
            selected = {null, null};
            
            settings = new FootnoteSettings ();
            
            this.build ();
            this.displayed_years = new GLib.List<int> ();
            
            /*check if db file exists, show welcome otherwise*/
            var db_file = File.new_for_path (Environment.get_user_data_dir ()+"/footnote/notes.db");
            var need_create = !db_file.query_exists ();
            if (need_create) {
                this.mainwindow.foreach ( (w) => this.mainwindow.remove (w) ); //clear childs
                this.mainwindow.add (this.welcome);
                this.mainwindow.show_all ();
            } else {
                open_db (false);
            }
            
            if (settings.window_width != 0 && settings.window_height != 0)
                this.mainwindow.resize (settings.window_width, settings.window_height);
            
            if (settings.view_mode == 1)
                this.view_mode.selected = 1;

			if (selected.notebook != null)
	            this.selected.notebook.view.miller_paned.position = settings.miller_handle_position;
            
            mainwindow.destroy.connect ( () => {
                foreach (Note n in deleted_notes) {
                    deleted_notes.remove (n);
                    //action_delete_note ();
                    n.notebook.delete_note (n);
                }
                
                settings.window_width  = mainwindow.get_allocated_width ();
                settings.window_height = mainwindow.get_allocated_height ();
                settings.sidebar_width = hmain.position;
                settings.view_mode     = this.view_mode.selected;
                settings.miller_handle_position = selected.notebook.view.miller_paned.position;
                
                Gtk.main_quit ();
            });
            
        }
        
        private void open_db (bool need_create, bool add_new=false) {
            try {
                db = new SQLHeavy.Database (Environment.get_user_data_dir ()+"/footnote/notes.db", 
                    SQLHeavy.FileMode.READ | SQLHeavy.FileMode.WRITE | SQLHeavy.FileMode.CREATE);
                if (need_create) {
                    db.execute ("CREATE TABLE notebooks (name TEXT, color INTEGER, id INTEGER PRIMARY KEY AUTOINCREMENT);");
                    db.execute ("CREATE VIRTUAL TABLE notes USING fts3(title TEXT, content BLOB, notebook INTEGER, bookmarked BOOL, date TEXT);");
                }
            } catch (SQLHeavy.Error e) { warning ("Could not create db, %s", e.message); }
            
            
            try {
                for (var res=db.execute ("SELECT name,color,id FROM 'notebooks';");!res.finished;res.next ()){
                    new Notebook.from_existing  (this, res.fetch_string (0), (uint32)res.fetch_int64 (1), res.fetch_int(2));
                }
            } catch (SQLHeavy.Error e) {
                warning ("Could not query the notebooks, %s", e.message);
                /*temporary warning for those who have upgraded*/
                var dlg = new Gtk.Dialog.with_buttons ("Problem", this.mainwindow, Gtk.DialogFlags.MODAL, Gtk.Stock.OK, Gtk.ResponseType.OK);
                var box = new Gtk.Box (Gtk.Orientation.HORIZONTAL, 0);
                ((Gtk.Box)dlg.get_content_area ()).add (box);
                
                var lbl = new Gtk.Label ("Footnote could not query the notebooks. If this happens after an update you either want to delete the old database at $home/.local/share/footnote/notes.db or, if you already have (very) important data in the database, <a href='https://bugs.launchpad.net/footnote/+filebug'>report a bug</a> asking for a converting tool, we will se what we can do.");
                lbl.wrap_mode = Pango.WrapMode.WORD;
                lbl.wrap = true;
                lbl.use_markup = true;
                lbl.margin_left = 12;
                
                box.margin = 12;
                box.pack_start (new Gtk.Image.from_stock (Gtk.Stock.DIALOG_ERROR, Gtk.IconSize.DIALOG), false);
                box.pack_start (lbl);
                dlg.show_all ();
                dlg.run ();
                dlg.destroy ();
            }
            
            if (this.notebooks.length () == 0)
                this.notebooks.append (new Footnote.Widgets.Notebook (this, _("Unsorted Notes"), 255, 0, 0));
            if (add_new)
                this.notebooks.append (new Footnote.Widgets.Notebook (this, _("Untitled"), 0, 255, 0));
            
            selected.notebook = notebooks.nth_data (0);
            selected.notebook.view.update_view ();
            
            this.note_view.foreach ( (w) => { w.hide (); });
            
            selected.notebook.view.show_all ();
            this.selected.notebook.view.miller_paned.hide ();
        }
        
        private void build () {
            
            this.mainwindow = new Gtk.Window ();
            
            /*actions*/
            this.main_actions = new Gtk.ActionGroup ("MainActionGroup"); /*Actions and UIManager*/
            this.main_actions.set_translation_domain ("footnote");
            this.main_actions.add_actions (main_entries, mainwindow);
            this.main_actions.add_toggle_actions (toggle_entries, mainwindow);
            
            this.ui = new Gtk.UIManager ();
            try {
                this.ui.add_ui_from_string (ui_string, -1);
            }
            catch (GLib.Error e) { error (e.message); }
            
            mainwindow.add_accel_group (ui.get_accel_group());
            ui.insert_action_group (main_actions, 0);
            ui.ensure_update ();
            
            
            this.undo_bar       = new Gtk.InfoBar.with_buttons (Gtk.Stock.UNDO, Gtk.ResponseType.CANCEL);
            this.welcome        = new Granite.Widgets.Welcome (_("Write things down"), _("Footnote can't seem to find your notes"));
            var vmain           = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            var tb              = new Gtk.Toolbar ();
            var separat         = new Gtk.ToolItem ();
            var search_item     = new Gtk.ToolItem ();
            var view_item       = new Gtk.ToolItem ();
            this.view_mode      = new Granite.Widgets.ModeButton ();
            this.search         = new Granite.Widgets.SearchBar (_("Type to Search..."));
            var export_nb_menu  = new Gtk.MenuItem.with_label (_("Export current notebook..."));
            var menu            = new Gtk.Menu ();
            var appmenu         = create_appmenu (menu);
            var add_new         = main_actions.get_action ("NewNote").create_tool_item () as Gtk.ToolButton;
            var fullscreen      = main_actions.get_action ("Fullscreen").create_menu_item () as Gtk.MenuItem;
            var sep2            = new Gtk.SeparatorToolItem ();
            this.hmain          = new Granite.Widgets.SidebarPaned ();
            this.edit_view      = new Footnote.Widgets.EditView (this);
            
            var left_box        = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            var left_scrolled   = new Gtk.ScrolledWindow (null, null);
            this.sidebar        = new ElementaryWidgets.SideBar ();
            this.notebooks_iter = this.sidebar.addItem (null, null, null, null, _("Notebooks"), null);
            this.bookmarks_iter = this.sidebar.addItem (null, null, null, null, _("Bookmarks"), null);
            
            var buttons_tb      = new Gtk.Toolbar ();
            this.add_button     = new Gtk.ToolButton (new Gtk.Image.from_icon_name ("list-add-symbolic", Gtk.IconSize.MENU),"");
            this.remove_button  = new Gtk.ToolButton (new Gtk.Image.from_icon_name ("list-remove-symbolic", Gtk.IconSize.MENU),"");
            this.edit_button    = new Gtk.ToolButton (new Gtk.Image.from_icon_name ("folder-documents-symbolic", Gtk.IconSize.MENU), "");
            
            this.right_box      = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            this.note_view      = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            var topbar          = new Gtk.Toolbar ();
            var years_item      = new Gtk.ToolItem ();
            this.years          = new Gtk.ComboBoxText ();
            
            
            ((Gtk.Container)undo_bar.get_content_area ()).add (new Gtk.Label (_("Note has been deleted.")));
            
            welcome.append ("document-import", _("Import"), _("Add notes from your hard drive"));
            welcome.append ("document-new", _("Create"), _("Make a new note"));
            welcome.append ("address-book-new", _("Create"), _("Create a new notebook"));
            
            //top
            separat.set_expand (true);
            search_item.add (search);
            menu.prepend (new Gtk.SeparatorMenuItem ());
            menu.prepend (fullscreen);
            menu.prepend (new Gtk.SeparatorMenuItem ());
            menu.prepend (export_nb_menu);
            
            view_mode.append (new Gtk.Image.from_icon_name ("view-filter-symbolic", Gtk.IconSize.MENU));
            view_mode.append (new Gtk.Image.from_icon_name ("view-column-symbolic", Gtk.IconSize.MENU));
            view_mode.valign = Gtk.Align.CENTER;
            view_mode.selected = 0;
            view_item.add (view_mode);
            
            tb.get_style_context  ().add_class  (Gtk.STYLE_CLASS_PRIMARY_TOOLBAR);
            tb.insert (add_new,     -1);
            tb.insert (view_item,   -1);
            tb.insert (sep2,        -1);
            tb.insert (separat,     -1);
            tb.insert (search_item, -1);
            tb.insert (appmenu,     -1);
            
            //left
            left_scrolled.add (this.sidebar);
            
            this.sidebar.get_style_context ().add_class ("sidebar");
            
            buttons_tb.icon_size = Gtk.IconSize.MENU;
            buttons_tb.get_style_context ().add_class ("inline-toolbar");
            buttons_tb.insert (add_button,    -1);
            buttons_tb.insert (remove_button, -1);
            buttons_tb.insert(edit_button,    -1);
            
            left_box.pack_start (left_scrolled, true, true, 0);
            left_box.pack_start (buttons_tb, false, false, 0);
            
            //right
            topbar.set_icon_size (Gtk.IconSize.MENU);
            topbar.toolbar_style = Gtk.ToolbarStyle.TEXT;
            topbar.get_style_context ().add_class ("secondary-toolbar");
            
            years.append ("-1", _("Any"));
            years.active_id = "-1";
            years_item.add (years);
            years_item.margin_right = 10;
            
            right_box.pack_start (topbar, false, true, 0);
            right_box.pack_start (note_view);
            
            hmain.get_style_context ().add_class ("sidebar-pane-separator");
            hmain.pack1 (left_box,  false, true);
            hmain.pack2 (right_box, true, true);
            hmain.position = settings.sidebar_width;
            
            vmain.pack_start (tb, false);
            vmain.pack_start (hmain);
            
            /*fill our date filter*/
            var all = new Gtk.RadioToolButton (null);
            all.label = _("All");
            all.toggled.connect ( () => {if (all.active)month = -1;selected.notebook.view.update_view ();});
            
            topbar.insert (years_item, -1);
            topbar.insert (all,        -1);
            topbar.child_set (all, "homogeneous", false);
            
            string [] months = {_("January"), _("February"), _("March"), _("April"), _("May"), 
                _("June"), _("July"), _("August"), _("September"), _("October"), _("November"), _("December")};
            for (var i=0;i<months.length;i++) {
                var w = new Gtk.RadioToolButton.from_widget (all);
                w.label = months[i];
                topbar.insert (w, -1);
                topbar.child_set (w, "homogeneous", false);
                var m = i+1;
                w.toggled.connect ( () => {if (w.active)month = m; selected.notebook.view.update_view ();});
            }
            
            this.mainwindow.title = program_name;
            this.mainwindow.icon_name = app_icon;
            this.mainwindow.set_default_size (640,480);
            this.mainwindow.add (vmain);
            this.mainwindow.show_all ();
            
            /**
             * events
             **/
            /*switch views*/
            this.view_mode.mode_changed.connect ( () => {
                this.selected.notebook.view.update_view ();
            });
            /*search on every keypress*/
            search.key_release_event.connect (() => {
                on_search_changed ();
                return false;
            });
            /*search icon pressed, start searching*/
            search.icon_press.connect ((entry, event) => {
                if (entry == Gtk.EntryIconPosition.SECONDARY)
                   on_search_changed ();
            });
            /*global notebook export*/
            export_nb_menu.activate.connect (()=>{
                new ExportDialog.export_notes (selected.notebook.notes, selected.notebook.title, this.mainwindow);
            });
            /*notebook selection changed*/
            sidebar.true_selection_change.connect ( (iter) => {
                
                var place = int.parse (sidebar.filter.get_model ().get_path (sidebar.convertToChild (iter)).to_string ().substring (0, 1));
                
                if (place == 0) { //notebooks
                    var a = selected.notebook.view.list_view.animate (Clutter.AnimationMode.EASE_OUT_QUAD, 
                        (this.view_mode.selected == 0)?400:1, y:selected.notebook.view.list_view.get_stage ().height);
                    
                    a.completed.connect ( () => {
                        var selected_item = (Footnote.Widgets.Notebook) this.sidebar.getObject (iter);
                        selected.note = selected_item.notes.nth_data(0);
                        switch_to_notebook (selected_item);
                    });
                }else if (place == 1) { //bookmarks
                    selected.note = (Footnote.Widgets.Note) sidebar.getObject (iter);
                    switch_to_notebook (selected.note.notebook);

                    if (view_mode.selected == 0) {
                        selected.notebook.view.open_note (selected.note);
                    }
                }
            });
            /*add a new notebook*/
            this.add_button.clicked.connect ( ()=>{
                var dlg     = new Granite.Widgets.PopOver ();
                var name    = new Gtk.Entry ();
                var ok      = new Gtk.Button.from_stock (Gtk.Stock.ADD);
                var b       = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
                var colb    = new Footnote.Widgets.ColorRow ();
                
                
                name.set_placeholder_text (_("Notebook name"));
                ok.set_sensitive(false);
                
                b.pack_start(new LLabel (_("Name")));
                b.pack_start (name);
                b.pack_start (new LLabel (_("Color")));
                b.pack_start (colb);
                b.pack_start (ok);
                
                name.key_release_event.connect ( (event)=>{
                    if (name.text != ""){
                        ok.set_sensitive(true);
                        if  (event.keyval == 65293){
                            ok.clicked ();
                        }
                    }else{
                        ok.set_sensitive(false);
                    }
                    return false;
                });
                ok.clicked.connect ( ()=> {
                    this.notebooks.append (new Footnote.Widgets.Notebook (this, name.text, 
                        (uint8)colb.color.red, (uint8)colb.color.green, (uint8)colb.color.blue));
                    this.remove_button.set_sensitive (true);
                    this.main_actions.get_action ("NewNote").set_sensitive (true);
                    dlg.destroy ();
                });
                ((Gtk.Box)dlg.get_content_area ()).add (b);
                dlg.show_all ();
                dlg.move_to_widget (this.add_button);
                dlg.present ();
                dlg.run ();
                dlg.destroy ();
            });
            /*editview was closed*/
            this.edit_view.close.connect ( (reverse) => {
                if ((reverse)?view_mode.selected != 0:view_mode.selected == 0) {
                    this.selected.notebook.view.toggle_edit (false);
                } else {
                    this.selected.notebook.view.miller_paned.remove (this.selected.notebook.view.miller_paned.get_child2 ());
                    this.selected.notebook.view.miller_paned.pack2 (this.selected.notebook.view.miller_scroll, false, true);
                    this.selected.notebook.view.miller_paned.show_all ();
                    this.selected.notebook.view.clutter.hide ();
                }
                topbar.show_all ();
            });
            /*editview was opened*/
            this.edit_view.open.connect ( () => {
                if (view_mode.selected == 0) {
                    this.selected.notebook.view.toggle_edit (true);
                    topbar.hide ();
                } else {
                    this.selected.notebook.view.miller_paned.remove (this.selected.notebook.view.miller_paned.get_child2 ());
                    this.selected.notebook.view.miller_paned.pack2 (this.edit_view, false, true);
                    this.selected.notebook.view.miller_paned.show_all ();
                    this.selected.notebook.view.clutter.hide ();
                }
            });
            /*edit current category*/
            this.edit_button.clicked.connect ( ()=>{
                selected.notebook.edit (this);
            });
            /*remove current notebook*/
            this.remove_button.clicked.connect ( ()=>{
                Gtk.TreePath path;
                Gtk.TreeIter iter;
                
                this.sidebar.get_cursor (out path, null);
                this.sidebar.get_model ().get_iter (out iter, path);
                
                var place = int.parse (sidebar.filter.get_model ().get_path (sidebar.convertToChild (iter)).to_string ().substring (0, 1));
                
                if (place == 0) { //notebooks
                    selected.notebook.del (this);
                }else if (place == 1) { //bookmarks
                    if (sidebar.getName (sidebar.convertToChild (iter)) != _("Search Term")) {
                        ((Note)sidebar.getObject (iter)).bookmarked.active = false;
                    }
                }
            });
            /*year changed*/
            years.changed.connect ( () => {
                this.year = int.parse (years.active_id);
                selected.notebook.view.update_view ();
            });
            /*welcome screen*/
            this.welcome.activated.connect ( (index) => {
                switch (index) {
                    case 0:
                        var f = new Gtk.FileChooserDialog (_("Open the database"), this.mainwindow, Gtk.FileChooserAction.OPEN, 
                            Gtk.Stock.CANCEL, Gtk.ResponseType.CANCEL, 
                            Gtk.Stock.OK, Gtk.ResponseType.OK);
                        f.filter = new Gtk.FileFilter ();
                        f.filter.add_mime_type ("application/x-sqlite3");
                        
                        if (f.run () == Gtk.ResponseType.OK) {
                            try {
                                f.get_file ().copy (File.new_for_path (Environment.get_user_data_dir ()+"/footnote/notes.db"), 0);
                                
                                this.open_db (false);
                            } catch (Error e) { warning (e.message); }
                            f.destroy ();
                        } else {
                            f.destroy ();
                            return;
                        }
                        break;
                    case 1:
                        this.open_db (true);
                        break;
                    default:
                        this.open_db (true, true);
                        break;
                }
                this.mainwindow.remove (this.welcome);
                this.mainwindow.add (vmain);
            });
            /*DnD*/
            Gtk.TargetEntry e = {"footnote_note", 0, 0};
            Gtk.drag_dest_set (this.sidebar, Gtk.DestDefaults.ALL, {e}, Gdk.DragAction.MOVE);
            this.sidebar.drag_data_received.connect ( (widget, ctx, x, y, sel, info, timestamp) => {
                Gtk.TreePath path;
                Gtk.TreeIter iter;
                this.sidebar.get_dest_row_at_pos (x, y, out path, null);
                this.sidebar.model.get_iter (out iter, path);
                
                selected.notebook.notes.remove (selected.note);
                selected.notebook.view.remove_note (selected.note);
                
                var dest_cat = (Footnote.Widgets.Notebook)this.sidebar.getObject (iter);
                dest_cat.notes.append (selected.note);
                dest_cat.view.add_note (selected.note);
                selected.note.notebook = dest_cat;
                
                selected.notebook.view.update_view ();
                try {
                    var q = db.prepare ("UPDATE 'notes' SET notebook=:cat WHERE rowid=:id");
                    q.set_int (":cat", selected.note.notebook.id);
                    q.set_int(":id", selected.note.id);
                    q.execute();
                } catch (SQLHeavy.Error e) { warning (e.message); }
                
                Gtk.drag_finish (ctx, true, false, timestamp);
            });
        }

        private void switch_to_notebook (Footnote.Widgets.Notebook notebook) {
            this.note_view.foreach ( (w) => w.hide () );
            
            if (this.edit_view.parent != null)
                this.edit_view.close ();
            
            notebook.view.miller_paned.position = selected.notebook.view.miller_paned.position;

            selected.notebook = notebook;

            selected.notebook.view.list_view.y = -selected.notebook.view.list_view.get_stage ().height;
            selected.notebook.view.list_view.animate (Clutter.AnimationMode.EASE_OUT_QUAD, 
                400, y:0.0f);
            selected.notebook.view.calculate_height ();
            
            selected.notebook.view.show_all ();
            if (view_mode.selected == 0)
                selected.notebook.view.miller_paned.hide ();
            else
                selected.notebook.view.clutter.hide ();
            selected.notebook.view.update_view ();

            selected.notebook.view.miller_noteviewer.open_note (selected.note);
        }
        
        private void on_search_changed () {
            foreach (Notebook c in notebooks) {
                c.view.filter = search.text;
            }
            selected.notebook.view.update_view ();
        }
        
        
        static const Gtk.ActionEntry[] main_entries = {
           { "Quit", Gtk.Stock.QUIT,
          /* label, accelerator */       N_("Quit"), "<Control>q",
          /* tooltip */                  N_("Quit"),
                                         Gtk.main_quit },
            
           { "NewNote", Gtk.Stock.NEW,
          /* label, accelerator */       N_("New Note"), "<Control>t",
          /* tooltip */                  N_("Create a new note"),
                                         action_new_note },
           { "DeleteNote", Gtk.Stock.CLOSE,
          /* label, accelerator */       N_("Delete Note"), "<Control>w",
          /* tooltip */                  N_("Move the note into the trash"),
                                         action_delete_note },
           { "Fullscreen", Gtk.Stock.FULLSCREEN,
          /* label, accelerator */       N_("Fullscreen"), "F11",
          /* tooltip */                  N_("Toggle fullscreen"),
                                         action_toggle_fullscreen }
           };
        static const Gtk.ToggleActionEntry[] toggle_entries = {
           { "SetBold",   Gtk.Stock.BOLD,
                                         N_("Set Bold"), "<Control>b",
                                         N_("Set the selected text to bold"),
                                         action_set_bold },
          { "StrikeThrough",   Gtk.Stock.STRIKETHROUGH,
                                         N_("Strike through"), "<Control>d",
                                         N_("Strike though the selected text"),
                                         action_strike_through },
          { "SetItalic",   Gtk.Stock.ITALIC,
                                         N_("Set Italic"), "<Control>i",
                                         N_("Set the selected text to italic"),
                                         action_set_italic },
          { "SetMarked",   Gtk.Stock.CLEAR,
                                         N_("Set Marked"), "<Control>m",
                                         N_("Mark the selected text"),
                                         action_set_marked }
        };
    }
}

public Footnote.FootnoteApp nt;

public void action_toggle_fullscreen () {
    if (nt.fullscreened) {
        nt.mainwindow.unfullscreen ();
        nt.fullscreened = false;
    } else {
        nt.mainwindow.fullscreen ();
        nt.fullscreened = true;
    }
}
public void action_new_note () {
    var n = new Note (nt, nt.selected.notebook);
    nt.selected.notebook.add_note (n);
    nt.edit_view.open_note (n);
}
public void action_delete_note () {
    nt.selected.notebook.delete_note (nt.selected.note);
}
public void action_set_bold () {
    nt.edit_view.current_note.tag ("bold", ((Gtk.ToggleAction)nt.main_actions.get_action ("SetBold")).active);
}
public void action_strike_through () {
    nt.edit_view.current_note.tag ("strikethrough", ((Gtk.ToggleAction)nt.main_actions.get_action ("StrikeThrough")).active);
}
public void action_set_italic () {
    nt.edit_view.current_note.tag ("italic", ((Gtk.ToggleAction)nt.main_actions.get_action ("SetItalic")).active);
}
public void action_set_marked () {
    nt.edit_view.current_note.tag ("marked", ((Gtk.ToggleAction)nt.main_actions.get_action ("SetMarked")).active);
}

public static void main (string [] args) {
    GtkClutter.init (ref args);
    
    nt = new Footnote.FootnoteApp ();
    
    Gtk.main ();
}
