
const string [] colors = { //tango color palette
    "fce94f", "edd400", "c4a000", //butter
    "fcaf3e", "f57900", "ce5c00", //orange
    "e9b96e", "c17d11", "8f5902", //chocolate
    "8ae234", "73d216", "4e9a06", //chameleon
    "729fcf", "3465a4", "204a87", //sky blue
    "ad7fa8", "75507b", "5c3566", //plum
    "ef2929", "cc0000", "a40000", //scarlet red
    "eeeeec", "d3d7cf", "babdb6", //aluminium
    "888a85", "555753", "2e3436"
};

namespace Footnote.Widgets {
    
    public class ColorRow : Gtk.Box {
        
        public static Gdk.Pixbuf get_color_dot_from_hex (string colorhex, int w, int h) {
            Gdk.RGBA c = {};
            c.parse ("#"+colorhex);
            
            return get_color_dot (c.red, c.green, c.blue, w, h);
        }
        
        public static Gdk.Pixbuf get_color_dot (double r, double g, double b,int w, int h) {
            
            var surface = new Cairo.ImageSurface (Cairo.Format.ARGB32, w, h);
            var ctx     = new Cairo.Context (surface);
            
            var grad = new Cairo.Pattern.linear (0, 0, 0, h);
            grad.add_color_stop_rgb (0, r, g, b);
            grad.add_color_stop_rgb (1, r*0.8, g*0.8, b*0.8);
            
            var grad2 = new Cairo.Pattern.radial (w/2, 0, 1, w/2, 0, w);
            grad2.add_color_stop_rgba (0, 1, 1, 1, 0.6);
            grad2.add_color_stop_rgba (1, 1, 1, 1, 0.0);
            
            ctx.set_line_width (1);
            
            Granite.Drawing.Utilities.cairo_rounded_rectangle (ctx, 0.5, 0.5, w-1, h-1, 3);
            ctx.set_source (grad);
            ctx.fill_preserve ();
            ctx.set_source (grad2);
            ctx.fill_preserve ();
            ctx.set_source_rgba (0, 0, 0, 0.3);
            ctx.stroke ();
            
            return Gdk.pixbuf_get_from_surface (surface, 0, 0, w, h);
        }
        
        Gtk.Image [] images;
        
        const int size = 15;
        
        string _color;
        public unowned string color_string {
            get { return _color; }
            set {_color = value; current.pixbuf = this.get_color_dot_from_hex (_color, this.size, this.size); }
        }
        
        public Gdk.Color color {
            private set { color = value; }
            get {
                Gdk.Color c;
                Gdk.Color.parse ("#"+color_string, out c);
                return c;
            }
        }
        
        Gtk.Image current;
        
        public ColorRow (int show=8) {
            
            var menu  = new Gtk.Menu ();
            var arrow = new Gtk.EventBox ();
            
            arrow.add (new Gtk.Arrow (Gtk.ArrowType.DOWN, Gtk.ShadowType.NONE));
            
            this._color  = "000000";
            this.current = new Gtk.Image.from_pixbuf (this.get_color_dot_from_hex (_color, 
                this.size, this.size));
            this.current.margin_right = 6;
            
            this.pack_start (current, false);
            
            this.images = new Gtk.Image [colors.length];
            
            for (var i=0;i<this.images.length;i++)
                this.images[i] = new Gtk.Image.from_pixbuf (this.get_color_dot_from_hex (colors[i], 
                    this.size, this.size));
            
            /*fill the box*/
            for (var i=0;i<show;i++) {
                var e = new Gtk.EventBox ();
                e.add (this.images [i]);
                this.pack_start (e, false);
                var idx = i;
                e.button_press_event.connect ( () => {
                    this.color_string = colors[idx];
                    return false;
                });
            }
            
            this.pack_start (arrow, false, false);
            
            /*fill the menu*/
            for (var i=show;i<this.images.length;i++) {
                var m = new Gtk.MenuItem ();
                m.add (this.images[i]);
                menu.append (m);
                var idx = i;
                m.activate.connect ( () => {
                    this.color_string = colors[idx];
                });
            }
            
            arrow.button_press_event.connect ( (e) => {
                menu.popup (null, null, (m, x, y, push_in) => {x=0;y=0;push_in=true;}, e.button, e.time);
                return true;
            });
            
            menu.attach_to_widget (arrow, (w,m) => {});
            menu.show_all ();
        }
    }
}
