
namespace Footnote.Widgets {
    
    public class EditView : Gtk.EventBox {
        
        public Gtk.TextView title;
        public Gtk.TextView content;
        public Gtk.Toolbar  editbar;
        
        public signal void close (bool reverse=false);
        public signal void open  ();
        
        public Note current_note;
        
        FootnoteApp app;
        
        public EditView (FootnoteApp app) {
            
            this.app = app;
            
            var container = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            
            this.visible_window = false;
            this.get_style_context ().add_class ("content-view");
            
            this.editbar     = new Gtk.Toolbar ();
            this.editbar.get_style_context ().add_class ("secondary-toolbar");
            
            this.title   = new Gtk.TextView ();
            this.content = new Gtk.TextView ();
            var content_scroll = new Gtk.ScrolledWindow (null, null);
            
            var buttons     = new Gtk.ButtonBox (Gtk.Orientation.HORIZONTAL);
            var cancel      = new Gtk.Button.with_label (_("Cancel"));
            var done        = new Gtk.Button.with_label (_("Done"));
            
            var redo        = new Gtk.ToolButton.from_stock (Gtk.Stock.REDO);
            var undo        = new Gtk.ToolButton.from_stock (Gtk.Stock.UNDO);
            var bold        = app.main_actions.get_action ("SetBold").create_tool_item () as Gtk.ToggleToolButton;
            var italic      = app.main_actions.get_action ("SetItalic").create_tool_item () as Gtk.ToggleToolButton;
            var strike      = app.main_actions.get_action ("StrikeThrough").create_tool_item () as Gtk.ToggleToolButton;
            var mark        = app.main_actions.get_action ("SetMarked").create_tool_item () as Gtk.ToggleToolButton;
            //var font_item   = new Gtk.ToolItem ();
            //var font        = new Gtk.FontButton ();
            
            //font_item.add (font);
            
            editbar.insert (undo,                         -1);
            editbar.insert (redo,                         -1);
            editbar.insert (new Gtk.SeparatorToolItem (), -1);
            editbar.insert (bold,                         -1);
            editbar.insert (italic,                       -1);
            editbar.insert (strike,                       -1);
            editbar.insert (new Gtk.SeparatorToolItem (), -1);
            editbar.insert (mark,                         -1);
            
            buttons.pack_start (cancel);
            buttons.pack_start (done);
            buttons.layout_style = Gtk.ButtonBoxStyle.END;
            buttons.spacing      = 24;
            buttons.margin       = 24;
            
            content_scroll.add (this.content);
            content_scroll.hscrollbar_policy = Gtk.PolicyType.NEVER;
            this.content.wrap_mode = Gtk.WrapMode.WORD_CHAR;
            this.content.modify_font (Pango.FontDescription.from_string ("12"));
            this.title.modify_font (Pango.FontDescription.from_string ("bold 15"));
            this.title.margin_top = 24;
            
            content_scroll.margin_left = content_scroll.margin_right = this.title.margin_left = 24;
            
            container.pack_start (this.editbar, false);
            container.pack_start (this.title,   false);
            container.pack_start (content_scroll);
            container.pack_start (new Gtk.Separator (Gtk.Orientation.HORIZONTAL), false);
            container.pack_start (buttons,       false);
            
            this.title.margin_bottom = 12;
            
            this.add (container);
            
            this.title.key_press_event.connect ( (e) => {
                if (e.keyval == Gdk.Key.Tab || e.keyval == Gdk.Key.Return) {
                    this.content.grab_focus ();
                    return true;
                }
                return false;
            });
            this.draw.connect ( (ctx) => {
                ctx.rectangle (0, 0, this.get_allocated_width (), this.get_allocated_height ());
                ctx.set_source_rgb (1, 1, 1);
                ctx.fill ();
                return false;
            });
            done.clicked.connect ( () => {
                current_note.title.label   = title.buffer.text;
                current_note.save ();
                this.close ();
            });
            cancel.clicked.connect ( () => {
            	if (current_note.is_brand_new == true)
            		action_delete_note ();
                this.close ();
            });
            title.buffer.changed.connect ( () => {
                if (title.buffer.text == "")
                    done.sensitive = false;
                else
                    done.sensitive = true;
            });
        }
        
        public void open_note (Note n) {
            if (this.parent == null)
                this.open (); //show the edit view
            title.buffer.text = n.title.label;
            content.buffer    = n.content.buffer;
            
            this.content.buffer.notify["cursor-position"].connect ( () => {
                Gtk.TextIter s;
                content.buffer.get_selection_bounds (out s, null);
                string [] tags = new string[s.get_tags ().length ()];
                for (var i=0;i<s.get_tags ().length ();i++) {
                    tags[i] = s.get_tags ().nth_data (i).name;
                }
                ((Gtk.ToggleAction)app.main_actions.get_action ("SetBold")).active = ("bold" in tags);
                ((Gtk.ToggleAction)app.main_actions.get_action ("SetItalic")).active = ("italic" in tags);
                ((Gtk.ToggleAction)app.main_actions.get_action ("StrikeThrough")).active = ("strikethrough" in tags);
                ((Gtk.ToggleAction)app.main_actions.get_action ("SetMarked")).active = ("marked" in tags);
            });
            
            current_note = n;
        }
        
        
    }
}
