//
//  Copyright (C) 2011-2012 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

namespace Footnote.Widgets {
    
    /*
     * minimal version of note, just the ui
     **/
    public class NoteViewer : Gtk.Box {
        
        public Gtk.Label    date;
        public Gtk.Label    title;
        public Gtk.TextView content;
        public Gtk.ToggleButton bookmarked;
        
        public Note         current_note;
        
        public NoteViewer (FootnoteApp app) {
            
            this.orientation = Gtk.Orientation.VERTICAL;
            
            var grid        = new Gtk.Grid ();
            this.title      = new LLabel ("");
            this.content    = new Gtk.TextView ();
            this.date       = new LLabel ("");
            var export      = new Gtk.Button.with_label (_("Export"));
            var edit        = new Gtk.Button.with_label (_("Edit"));
            var del         = new Gtk.Button.with_label (_("Delete"));
            this.bookmarked = new Gtk.ToggleButton ();
            var buttons     = new Gtk.Box (Gtk.Orientation.HORIZONTAL, 12);
            
            this.get_style_context ().add_class ("content-view");
            
            this.title.selectable = true;
            this.title.modify_font (Pango.FontDescription.from_string ("bold 15"));
            
            this.date.modify_font (Pango.FontDescription.from_string ("10"));
            this.date.margin_top = this.date.margin_bottom = 6;
            this.date.selectable = true;
            
            this.content.modify_font (Pango.FontDescription.from_string ("12"));
            this.content.editable = false;
            this.content.wrap_mode = Gtk.WrapMode.WORD_CHAR;
            this.content.valign = Gtk.Align.START;
            this.content.buffer.create_tag ("bold", "weight", Pango.Weight.BOLD);
            this.content.buffer.create_tag ("italic", "style", Pango.Style.ITALIC);
            this.content.buffer.create_tag ("strikethrough", "strikethrough", true);
            this.content.buffer.create_tag ("marked", "background", "#ffff00");
            this.content.vexpand = true;
            
            this.bookmarked.halign = Gtk.Align.END;
            this.bookmarked.valign = Gtk.Align.CENTER;
            this.bookmarked.add (new Gtk.Image.from_icon_name ("bookmark-new", Gtk.IconSize.BUTTON));
            
            buttons.hexpand = true;
            buttons.pack_start (export,   false);
            buttons.pack_start (new Gtk.Label (""));
            buttons.pack_start (edit,  false);
            buttons.pack_start (del,     false);
            buttons.margin_top = 12;
            buttons.margin_bottom = 12;
            
            grid.margin = 12;
            grid.attach (this.title,      0, 0, 1, 1);
            grid.attach (this.date,       0, 1, 1, 1);
            grid.attach (this.bookmarked, 1, 0, 1, 2);
            grid.attach (this.content,    0, 2, 2, 1);
            grid.attach (buttons,         0, 3, 2, 1);
            
            this.pack_start (grid);
            this.pack_start (new Gtk.Separator (Gtk.Orientation.HORIZONTAL), false);
            
            /**
             * Events
             **/
            /*ask the app to edit the note*/
            edit.clicked.connect ( () => {
                app.edit_view.open_note (current_note);
            });
            /*export clicked, show contractor dialog*/
            export.clicked.connect ( () => {
                var dialog = new Gtk.Dialog.with_buttons ("Export", app.mainwindow, Gtk.DialogFlags.MODAL,
                     Gtk.Stock.CANCEL, Gtk.ResponseType.REJECT, Gtk.Stock.OK, Gtk.ResponseType.ACCEPT);
                var contr = new Granite.Widgets.ContractorView (current_note.save_temp_file (), "text");
                contr.name_blacklist ({"Brasero"});
                contr.margin = 12;
                ((Gtk.Box)dialog.get_content_area ()).add (contr);
                
                dialog.show_all ();
                if (dialog.run () == Gtk.ResponseType.ACCEPT)
                    contr.run_selected ();
                dialog.destroy ();
            });
            /*delete note*/
            del.clicked.connect ( () => {
                var dlg  = new Granite.Widgets.PopOver ();
                var cont = new Gtk.Grid ();
                var yes  = new Gtk.Button.with_label (_("Delete"));
                
                yes.get_style_context ().add_class ("noundo");
                
                cont.attach (new Gtk.Label (_("Sure?")), 0, 0, 1, 1);
                cont.attach (yes, 0, 1, 1, 1);
                ((Gtk.Container)dlg.get_content_area ()).add (cont);
                
                yes.clicked.connect ( () => {
                    app.selected.note = this.current_note; //just make sure..
                    action_delete_note ();
                    dlg.destroy ();
                });
                
                dlg.show_all ();
                dlg.move_to_widget (del);
                dlg.present ();
                dlg.run ();
                dlg.destroy ();
            });
            this.bookmarked.toggled.connect ( () => {
                current_note.bookmarked.active = this.bookmarked.active;
                
                this.bookmarked.foreach ( (w) => this.bookmarked.remove (w) );
                if (this.bookmarked.active) {
                    this.bookmarked.add (new Gtk.Image.from_icon_name ("help-about", Gtk.IconSize.BUTTON));
                } else {
                    this.bookmarked.add (new Gtk.Image.from_icon_name ("bookmark-new", Gtk.IconSize.BUTTON));
                }
                this.bookmarked.show_all ();
            });
        }
        
        
        public void open_note (Note n) {
            this.current_note = n;
            this.content.buffer = n.content.buffer;
            this.title.label = n.title.label;
            this.date.label = n.date.label;
            this.bookmarked.active = n.bookmarked.active;
        }
    }
    
    /**
     * Display a (filtered) list of Notes for the currently
     * selected Category.
     */
    public class NoteView : Gtk.Box {
        
        public string filter;
        
        private FootnoteApp app;
        
        public Gtk.ScrolledWindow  clutter;
        public GtkClutter.Embed    stage;
        public GtkClutter.Actor    edit_actor;
        public Clutter.Box         list_view;
        public Gtk.TreeView        miller_view;
        public Gtk.ListStore       miller_store;
        public Gtk.TreeModelFilter miller_filter;
        public Gtk.Paned           miller_paned;
        public Gtk.ScrolledWindow  miller_scroll;
        public NoteViewer          miller_noteviewer;
        
        public GtkClutter.Actor    undo_bar;
        
        private int current; //currently used view
        
        /**
         * Constructor
         */
        public NoteView (FootnoteApp app) {
            this.app           = app;
            this.filter        = "";
            this.orientation   = Gtk.Orientation.VERTICAL;
            this.current       = 0;
            
            this.undo_bar      = new GtkClutter.Actor ();
            this.clutter       = new Gtk.ScrolledWindow (null, null);
            this.stage         = new GtkClutter.Embed ();
            this.clutter.add_with_viewport (this.stage);
            this.list_view     = new Clutter.Box (new Clutter.BoxLayout ());
            this.edit_actor    = new GtkClutter.Actor ();
            
            this.miller_paned  = new Gtk.Paned (Gtk.Orientation.HORIZONTAL);
            this.miller_store  = new Gtk.ListStore (2, typeof (string), typeof (Note));
            this.miller_filter = new Gtk.TreeModelFilter (this.miller_store, null);
            this.miller_view   = new Gtk.TreeView.with_model (this.miller_filter);
            this.miller_scroll = new Gtk.ScrolledWindow (null, null);
            this.miller_noteviewer = new NoteViewer (app);
            var miller_view_scroll = new Gtk.ScrolledWindow (null, null);
            
            this.stage.get_stage ().add_actor (this.undo_bar);
            this.undo_bar.add_constraint (new Clutter.BindConstraint (this.stage.get_stage (), Clutter.BindCoordinate.WIDTH, 0));
            this.undo_bar.height = 38;
            this.undo_bar.y = -38;
            
            
            this.clutter.shadow_type = Gtk.ShadowType.NONE;
            ((Clutter.BoxLayout)this.list_view.layout_manager).vertical = true;
            this.list_view.background_color = {255, 255, 255, 255};
            this.stage.get_stage ().add_actor (this.list_view);
            this.list_view.add_constraint (new Clutter.BindConstraint (this.stage.get_stage (), Clutter.BindCoordinate.WIDTH,  0));
            this.list_view.add_constraint (new Clutter.BindConstraint (this.stage.get_stage (), Clutter.BindCoordinate.HEIGHT, 0));
            
            
            var renderer = new Gtk.CellRendererText ();
            renderer.ellipsize = Pango.EllipsizeMode.END;
            renderer.set_padding (0, 5);
            this.miller_view.insert_column_with_attributes (-1, "", renderer, "markup", 0);
            this.miller_view.headers_visible = false;
            
            var css = new Gtk.CssProvider ();
            try {
                css.load_from_data ("*{background-color:alpha (#fff, 0);}", -1);
            } catch (Error e) { warning (e.message); }
            
            this.miller_scroll.add_with_viewport (this.miller_noteviewer);
            this.miller_scroll.get_child ().get_style_context ().add_provider (css, 20000);
            
            /*white bgs*/
            this.clutter.get_child ().draw.connect ( (ctx) => {
                ctx.rectangle (0, 0, this.get_allocated_width (), this.get_allocated_height ());
                ctx.set_source_rgb (1, 1, 1);
                ctx.fill ();
                return false;
            });
            this.miller_scroll.draw.connect ( (ctx) => {
                ctx.rectangle (0, 0, this.get_allocated_width (), this.get_allocated_height ());
                ctx.set_source_rgb (1, 1, 1);
                ctx.fill ();
                return false;
            });
            
            this.miller_view.cursor_changed.connect ( () => {
                if (this.miller_view.model == null)
                    return;
                
                Gtk.TreePath path;
                Gtk.TreeIter iter;
                Note note;
                
                this.miller_view.get_cursor (out path, null);
                if (path == null)
                    return;
                this.miller_filter.get_iter (out iter, path);
                this.miller_filter.get (iter, 1, out note);
                
                if (this.miller_paned.get_child2 () == app.edit_view) {
                    this.miller_paned.remove (this.miller_paned.get_child2 ());
                    this.miller_paned.pack2 (this.miller_scroll, false, true);
                }
                
                this.miller_noteviewer.open_note (note);
            });
            
            this.miller_filter.set_visible_func ( (model, it) => {
                Note n;
                model.get (it, 1, out n);
                
                if (n == null || n.deleted)
                    return false;
                
                if ((
                    (app.month == -1 || app.month == n.month) && 
                    (app.year  == -1 || app.year  == n.year )) &&
                    (filter == "" || 
                     filter.down () in n.title.label.down () || 
                     filter.down () in n.content.buffer.text.down ())) {
                         return true;
                     }
                return false;
            });
            
            this.edit_actor.add_constraint (new Clutter.BindConstraint (this.stage.get_stage (), Clutter.BindCoordinate.WIDTH,  0));
            //this.edit_actor.add_constraint (new Clutter.BindConstraint (this.stage.get_stage (), Clutter.BindCoordinate.HEIGHT, 0));
            this.edit_actor.rotation_angle_x = -120.0f;
            this.edit_actor.x = 0.0f;
            this.edit_actor.y = 0.0f;
            this.edit_actor.reactive = true;
            
            this.stage.get_stage ().add_child (this.edit_actor);
            ((Clutter.Stage)this.stage.get_stage ()).color = {255, 255, 255, 255};
            
            miller_view_scroll.add (this.miller_view);
            this.miller_paned.pack1 (miller_view_scroll, true, false);
            this.miller_paned.pack2 (this.miller_scroll, false, true);
            
            this.pack_start (this.clutter);
            this.pack_start (this.miller_paned);
            
            this.show_all ();
            this.miller_paned.hide ();
        }
        
        /*
         * only for list view
         **/
        public void toggle_edit (bool show) {
            if (show) {
                this.stage.height_request = 0;
                ((Gtk.Container)this.edit_actor.get_widget ()).add (app.edit_view);
                this.edit_actor.get_widget ().show_all ();
                this.edit_actor.raise_top ();
                this.edit_actor.show ();
                this.edit_actor.animate (Clutter.AnimationMode.EASE_OUT_CUBIC, 300, rotation_angle_x:0.0f);
                this.edit_actor.height = app.note_view.get_allocated_height ();
            } else {
                var a = this.edit_actor.animate (Clutter.AnimationMode.EASE_OUT_CUBIC, 300, rotation_angle_x:-120.0f);
                a.completed.connect ( () => {
                    this.edit_actor.hide ();
                    ((Gtk.Container)this.edit_actor.get_widget ()).remove (app.edit_view);
                    this.calculate_height ();
                });
            }
        }
        
        /*
         * reassign the height request of the container
         **/
        public void calculate_height () {
            float sum = 100.0f;
            for (var i=0;i<this.list_view.get_children ().length ();i++)
                sum += this.list_view.get_children ().nth_data (i).height;
            this.stage.height_request = (int)sum;
        }
        
        /**
         * Add a note to the view
         */
        public void add_note (Note n, int pos=0) {
            this.list_view.insert_child_at_index (n, pos);
            this.update_view ();
            
            n.add_constraint (new Clutter.BindConstraint (this.stage.get_stage (), Clutter.BindCoordinate.WIDTH, 0));
            ((Clutter.BoxLayout)this.list_view.layout_manager).set_fill (n, true, false);
            
            this.miller_store.prepend (out n.miller_iter);
            this.miller_store.set (n.miller_iter, 0, n.get_miller_label (), 1, n);
            
            this.calculate_height ();
        }
        
        /**
         * Remove a note from the view
         */
        public void remove_note (Note n) {
            var a = n.animate (Clutter.AnimationMode.EASE_OUT_QUAD, 1000, margin_left:n.get_stage ().width, height:0.0f);
            a.completed.connect ( () => this.list_view.remove_child (n) );
            this.update_view ();
            
            calculate_height ();
        }
        
        /**
         * Update the view, loop through ArrayList of Notes and
         * if filter isn't empty, only show notes containing the
         * filter string (e.g. in title, text or as a tag).
         */
        public void update_view () {
            /*switch view if necessary*/
            if (this.current != app.view_mode.selected) {
                if (app.edit_view.parent != null)
                    app.edit_view.close (true);
                if (app.view_mode.selected == 1) {
                    this.show_all ();
                    this.clutter.hide ();
                    this.miller_view.set_cursor (new Gtk.TreePath.from_string ("0"), null, false);
                } else {
                    this.show_all ();
                    this.miller_paned.hide ();
                }
                this.current = app.view_mode.selected;
            }
            
            if (app.view_mode.selected == 1)
                this.miller_filter.refilter ();
            else {
                var shown = 0;
                if (app.selected.notebook != null) {
                    foreach (Note n in app.selected.notebook.notes) {
                        if ((
                            (app.month == -1 || app.month == n.month) && 
                            (app.year  == -1 || app.year  == n.year )) &&
                            (filter == "" || 
                             filter.down () in n.title.label.down () || 
                             filter.down () in n.content.buffer.text.down ())) {
                                if (!n.visible)
                                    n.show ();
                                shown ++;
                        } else {
                            if (n.visible)
                                n.hide ();
                        }
                    }
                }
            }
        }

        /**
         * Scroll to a certain note in the list.
         */
        public void open_note (Footnote.Widgets.Note n) {
            if (app.view_mode.selected == 0) {
                // Scroll to the item in the list.
                int dest_x;
                int dest_y;
                n.title.translate_coordinates((Gtk.Widget)stage, 0, 0, out dest_x, out dest_y);
                clutter.vadjustment.value = dest_y;
            } else {
                // Show the note in the noteviewer.
                this.miller_noteviewer.open_note (n);
            }
        }
    }
}
